@@ -136,6 +136,19 @@ struct bfs_expr *bfs_expr_new(bfs_eval_fn *eval_fn, size_t argc, char **argv) {
 	expr->successes = 0;
 	expr->elapsed.tv_sec = 0;
 	expr->elapsed.tv_nsec = 0;
+
+	// Prevent bfs_expr_free() from freeing uninitialized pointers on error paths
+	if (bfs_expr_has_children(expr)) {
+		expr->lhs = NULL;
+		expr->rhs = NULL;
+	} else if (eval_fn == eval_exec) {
+		expr->exec = NULL;
+	} else if (eval_fn == eval_fprintf) {
+		expr->printf = NULL;
+	} else if (eval_fn == eval_regex) {
+		expr->regex = NULL;
+	}
+
 	return expr;
 }
 
@@ -3189,7 +3202,7 @@ static struct bfs_expr *parse_version(struct parser_state *state, int arg1, int
 typedef struct bfs_expr *parse_fn(struct parser_state *state, int arg1, int arg2);
 
 /**
- * An entry in the parse table for literals.
+ * An entry in the parse table for primary expressions.
  */
 struct table_entry {
 	char *arg;
@@ -3201,7 +3214,7 @@ struct table_entry {
 };
 
 /**
- * The parse table for literals.
+ * The parse table for primary expressions.
  */
 static const struct table_entry parse_table[] = {
 	{"--", T_FLAG},
@@ -3358,11 +3371,11 @@ static const struct table_entry *table_lookup_fuzzy(const char *arg) {
 }
 
 /**
- * LITERAL : OPTION
+ * PRIMARY : OPTION
  *         | TEST
  *         | ACTION
  */
-static struct bfs_expr *parse_literal(struct parser_state *state) {
+static struct bfs_expr *parse_primary(struct parser_state *state) {
 	// Paths are already skipped at this point
 	const char *arg = state->argv[0];
 
@@ -3423,7 +3436,7 @@ unexpected:
  * FACTOR : "(" EXPR ")"
  *        | "!" FACTOR | "-not" FACTOR
  *        | "-exclude" FACTOR
- *        | LITERAL
+ *        | PRIMARY
  */
 static struct bfs_expr *parse_factor(struct parser_state *state) {
 	if (skip_paths(state) != 0) {
@@ -3489,7 +3502,7 @@ static struct bfs_expr *parse_factor(struct parser_state *state) {
 
 		return new_unary_expr(eval_not, factor, argv);
 	} else {
-		return parse_literal(state);
+		return parse_primary(state);
 	}
 }
 
